use crate::geometry::{Point, Ray, Transform, Vector};

#[derive(Debug, Clone)]
pub struct Camera {
    /// camera to world transform
    transform: Transform,
}

impl Camera {
    pub fn new(eye: Point, center: Point, up: Vector, fov: f32) -> Self {
        let f = (center - eye).length();
        let tan = (fov / 2.0).to_radians().tan();
        let transform =
            Transform::look_at(eye, center, up).inverse() * Transform::scale(tan * f, tan * f, f);

        Self { transform }
    }

    pub fn ray(&self, x: f32, y: f32) -> Ray {
        let origin = self.transform * Point::ORIGIN;
        let direction = self.transform * Vector::new(x, y, 1.0);
        Ray::new(origin, direction)
    }
}
